use super::*;
use std::cell::RefCell;

pub type MemoryLayout    = Vec<Layout>;
pub type MemoryLayoutMut = Vec<LayoutMut>;

macro_rules! declare_memory_layout {
    ($($val:ident),*) => {
        /*
            Creates Layout(Mut)
            - add $($val),* to toggle mut
            - add $($val:camel),* inside paste::item to make NameMut or Name
        */
        paste::item! {
            pub struct [< DefaultLayout$($val:camel),* >] {
            }
            
            impl [< DefaultLayout$($val:camel),* >] {
                pub fn new(parent: &$($val),* CDDesc) -> Vec<[< Layout$($val:camel),* >]> {
                    let mut layout = Vec::new();
                    
                    layout.push([< Layout$($val:camel),* >]::SystemArea(parent.system_area.clone()));
                    layout.push([< Layout$($val:camel),* >]::PVD(parent.pvd.clone()));
                    layout.push(([< Layout$($val:camel),* >]::PathTables));

                    [< add_dir_and_subdir $(_$val),* >](&mut layout, parent.root.clone());
                    layout
                }
            }

            pub enum [< Layout$($val:camel),* >] {
                SystemArea(Rc<RefCell<SystemArea>>),
                PVD(Rc<RefCell<PrimaryVolumeDescriptor>>),
                PathTables,
                Directory(Rc<RefCell<Directory>>),
                File(Rc<RefCell<File>>)
            }           

            fn [< add_dir_and_subdir $(_$val),* >]<'a>(layout: &mut Vec<[< Layout$($val:camel),* >]>, dir: Rc<RefCell<Directory>>) {
                layout.push([< Layout$($val:camel),* >]::Directory(dir.clone()));
                
                let $($val),* dir = dir.[< borrow $(_$val),* >]();
                for file in dir.files.[< iter$(_$val),* >]() {
                    layout.push([< Layout$($val:camel),* >]::File(file.clone()));
                }

                for dir in dir.dirs.[< iter$(_$val),* >]() {
                    [< add_dir_and_subdir $(_$val),* >](layout, dir.clone());
                }
            }
        }
    };
}

declare_memory_layout!(mut);
declare_memory_layout!();